/*
 * Copyright (c) 2010 Ingres Corp.
 */

#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <math.h>

EXEC SQL INCLUDE SQLCA;

static void usage(char *msg)
{
	if (msg)
            fprintf(stderr, "%s\n", msg);
	fprintf( stderr, "TPC-H ACID Transaction with ROLLBACK\n\n");
	fprintf( stderr, "usage: acidtrans_rb -b <dbms> [-s <n>] < <key file>\n");
	fprintf( stderr, "  -s <n>   sleep n seconds prior to commit\n\n");
    fprintf( stderr, "\te.g. acidkeys -b sf1 | acidtrans_rb -b sf1\n");
	exit(0);
}

int main( int argc, char **argv)
{
	EXEC SQL BEGIN DECLARE SECTION;
		char *database=NULL;
        long long o_key;
        double tax, disc;
        long long quantity;
        double ototal, extprice;
        long long pkey, skey;
        long long l_key;
        int delta;
        double new_ototal;
        double rprice;
	EXEC SQL END DECLARE SECTION;

    char c;
	struct timeval tp;
	struct tm* ptm;
	char time_string[40];

    int delay = 0;
 
	extern char *optarg;
    extern int optind, optopt;

	while ((c = getopt(argc, argv, ":b:s:")) != -1)
		switch (c) {
		case 'b':
			database = optarg;
			break;
		case 's':
			delay = atoi(optarg);
			break;
		case ':':  /* -b without arguments */
			usage("option requires an argument");
			break;
		case '?':
			usage("unknown option");
		}	  

	if (database == NULL)
		usage("no database specified");

	if (delay < 0)
		usage("-s must be greater than or equal to 0");

	gettimeofday(&tp, NULL);
	ptm = localtime(&tp.tv_sec);
	strftime(time_string, sizeof(time_string), "%Y-%m-%d %H:%M:%S", ptm);
	printf("=========================================================\n");
	printf("TPC-H ACID Program started at: %s.%06ld\n", time_string, tp.tv_usec);
	printf("=========================================================\n");

	EXEC SQL WHENEVER SQLERROR STOP;
	EXEC SQL CONNECT TO :database;
    EXEC SQL SET AUTOCOMMIT OFF;

    while (scanf("%lld %lld %d\n", &o_key, &l_key, &delta) == 3) {
        EXEC SQL SELECT O_TOTALPRICE
		into :ototal
		FROM ORDERS WHERE O_ORDERKEY = :o_key;

        EXEC SQL SELECT L_QUANTITY, L_EXTENDEDPRICE, L_PARTKEY,
        L_SUPPKEY, L_TAX, L_DISCOUNT, TRUNC(L_EXTENDEDPRICE/L_QUANTITY, 2)
		INTO :quantity, :extprice, :pkey, :skey, :tax, :disc, :rprice
        FROM LINEITEM WHERE L_ORDERKEY = :o_key AND L_LINENUMBER = :l_key;

//ACID CODE BEGIN
		gettimeofday(&tp, NULL);
		ptm = localtime(&tp.tv_sec);
		strftime(time_string, sizeof(time_string), "%Y-%m-%d %H:%M:%S", ptm);

		printf("===================================================================\n");
		printf("TPC-H ACID Txn with ROLLBACK started at: %s.%06ld\n", time_string, tp.tv_usec);
		printf("===================================================================\n");
        printf("Input data:\n");
        printf("O_KEY = %lld\n", o_key);
        printf("L_KEY = %lld\n", l_key);
        printf("DELTA = %d\n\n", delta);

		printf("O_TOTALPRICE from ORDERS before ROLLBACK: %f\n\n", ototal);
		printf("Data from LINEITEM before changes:\n");
		printf("L_QUANTITY = %lld\n", quantity);
		printf("L_EXTENDEDPRICE = %f\n", extprice);
		printf("L_PARTKEY = %lld\n", pkey);
		printf("L_SUPPKEY = %lld\n", skey);
		printf("L_TAX = %f\n", tax);
		printf("L_DISCOUNT = %f\n\n", disc);
//ACID CODE END

		EXEC SQL SELECT TRUNC( TRUNC(L_EXTENDEDPRICE * (1.0 - L_DISCOUNT), 2) * (1.0 + L_TAX),2 )
		INTO :new_ototal
		FROM LINEITEM WHERE L_ORDERKEY = :o_key AND l_linenumber = :l_key;

		// Update ORDERS a first time to remove old LINEITEM from total price
		EXEC SQL UPDATE ORDERS
		SET O_TOTALPRICE = O_TOTALPRICE - decimal(:new_ototal,12,2)
		WHERE O_ORDERKEY = :o_key;

		// Update LINEITEM to increase quantity
		EXEC SQL UPDATE LINEITEM
		SET L_EXTENDEDPRICE = L_EXTENDEDPRICE + TRUNC(TRUNC(L_EXTENDEDPRICE/L_QUANTITY, 2) * :delta, 2 ),
		L_QUANTITY = L_QUANTITY + :delta
		WHERE L_ORDERKEY = :o_key AND L_LINENUMBER = :l_key;

		EXEC SQL SELECT TRUNC( TRUNC(L_EXTENDEDPRICE * (1.0 - L_DISCOUNT), 2) * (1.0 + L_TAX),2 )
		INTO :new_ototal
		FROM LINEITEM WHERE L_ORDERKEY = :o_key AND l_linenumber = :l_key;

		// Update ORDERS a second time to add updated LINEITEM to total price
		EXEC SQL UPDATE ORDERS
		SET O_TOTALPRICE = O_TOTALPRICE + decimal(:new_ototal,12,2)
		WHERE O_ORDERKEY = :o_key;

		// Insert into HISTORY records of transaction
        EXEC SQL INSERT INTO HISTORY
        VALUES( :pkey, :skey, :o_key, :l_key, :delta, timestamp(date('now')));

//ACID CODE BEGIN
		if (delay) {
			creat("waiting", 0777);
    		gettimeofday(&tp, NULL);
    		ptm = localtime(&tp.tv_sec);
    		strftime(time_string, sizeof(time_string), "%Y-%m-%d %H:%M:%S", ptm);

			printf(">> TPC-H ACID Transaction. Start of SLEEP: %s.%06ld\n\n", time_string, tp.tv_usec);
			fflush(NULL);

			sleep(delay);

    		gettimeofday(&tp, NULL);
    		ptm = localtime(&tp.tv_sec);
    		strftime(time_string, sizeof(time_string), "%Y-%m-%d %H:%M:%S", ptm);

			printf(">> TPC-H ACID Transaction. End of SLEEP: %s.%06ld\n\n", time_string, tp.tv_usec);
			fflush(NULL);
		}
//ACID CODE END

        EXEC SQL ROLLBACK;

//ACID CODE BEGIN
        EXEC SQL SELECT O_TOTALPRICE
		INTO :ototal
		FROM ORDERS WHERE O_ORDERKEY = :o_key;

        EXEC SQL SELECT L_QUANTITY, L_EXTENDEDPRICE, L_PARTKEY, L_SUPPKEY, L_TAX, L_DISCOUNT
		INTO :quantity, :extprice, :pkey, :skey, :tax, :disc
        FROM LINEITEM WHERE L_ORDERKEY = :o_key AND L_LINENUMBER = :l_key;

		printf("O_TOTALPRICE from ORDERS after ROLLBACK: %f\n\n", ototal);

		printf("Data from LINEITEM after ROLLBACK:\n");
		printf("L_QUANTITY = %lld\n", quantity);
		printf("L_EXTENDEDPRICE = %f\n", extprice);
		printf("L_PARTKEY = %lld\n", pkey);
		printf("L_SUPPKEY = %lld\n", skey);
		printf("L_TAX = %f\n", tax);
		printf("L_DISCOUNT = %f\n\n", disc);

		gettimeofday(&tp, NULL);
		ptm = localtime(&tp.tv_sec);
		strftime(time_string, sizeof(time_string), "%Y-%m-%d %H:%M:%S", ptm);

		printf("=================================================================\n");
		printf("TPC-H ACID Txn with ROLLBACK Ended at: %s.%06ld\n", time_string, tp.tv_usec);
		printf("=================================================================\n");
//ACID CODE END

        printf("%15s %10s %6s %6s %15s %15s\n",
               "rprice", "quantity", "tax", "disc", "extprice", "ototal");
        printf("%15.2f %10lld %6.2f %6.2f %15.2f %15.2f\n",
                rprice, quantity, tax, disc, extprice, ototal);
        fflush(NULL);
    }
    EXEC SQL DISCONNECT;

	gettimeofday(&tp, NULL);
	ptm = localtime(&tp.tv_sec);
	strftime(time_string, sizeof(time_string), "%Y-%m-%d %H:%M:%S", ptm);
	printf("=======================================================\n");
	printf("TPC-H ACID Program ended at: %s.%06ld\n", time_string, tp.tv_usec);
	printf("=======================================================\n");

    return(0);
}
