#!/bin/bash

#
# Copyright (c) 2010 Ingres Corp.
#

TOPDIR="`cat .topdir`"
IVWDIR="$TOPDIR/scripts/ingres_vectorwise"

usage()
{
cat << EOF
usage: $0 -d <db> [-s <n>] [-t <n>]

The script runs TPC-H ACID Durability Tests

OPTIONS:
    -h              show this message
    -d  <database>  database name
    -s  <number>    number of streams (default=2)
    -t  <number>    number of transactions (default=200)    
EOF
}

DATABASE=
STREAMS=2
NTRAN=200
while getopts "d:s:t:" opt; do
    case $opt in
        d)
            DATABASE=$OPTARG
            ;;
        s)
        	STREAMS=$OPTARG
        	;;
        t)
        	NTRAN=$OPTARG
        	;;            
        ?)
            usage
            exit 1
            ;;
    esac
done

if [ -z "$DATABASE" ]; then
	usage
	exit 1
fi

if [ $STREAMS -lt 2 ]; then
	echo "Error: Number of streams must be >= 2"
	exit 1
fi

if [ $NTRAN -lt 200 ]; then
	echo "Error: Number of transactions must be >= 200"
	exit 1
fi

echo "======================="
echo "TPC-H Durability Tests"
echo "======================="
echo "Test Started at: `date`"
echo

# drop and create the HISTORY table
echo "Drop the HISTORY table:"
echo "drop table HISTORY;\p\g" | sql $DATABASE
echo "Create the HISTORY table:"
cat $IVWDIR/acid/create_history.sql | sql $DATABASE

# create directory to store test-related data
#
OUTPUT_DIR="$IVWDIR/acid/dur/output"
mkdir -p $OUTPUT_DIR

# generate NTRAN random numbers for each stream in STREAMS
#
for ((i=0;i<${STREAMS};i++)); do
	echo "creating random numbers for stream $i..."
	$IVWDIR/acid/acidkeys -b $DATABASE -c $NTRAN > $OUTPUT_DIR/stream${i}_rnd
	echo "random numbers for stream $i stored in $OUTPUT_DIR/stream${i}_rnd"
done

###################
# STEP 1
###################
echo
echo ">>> STEP 1"
echo

# Obtain 10 ORDERKEYs of each stream and check consistency condition 
#
for ((i=0;i<${STREAMS};i++)); do
	KEYS=`head -n 10 $OUTPUT_DIR/stream${i}_rnd | awk '{printf "%d ", $1}'`
		
	for j in $KEYS; do
		$IVWDIR/acid/cons/consist_condition -b $DATABASE -o ${j} >> $OUTPUT_DIR/step1_stream${i}_output		
	done
	echo "	output for stream ${i} in $OUTPUT_DIR/step1_stream${i}_output"
done

echo
echo "STEP 1 COMPLETED."
echo

###################
# STEP 2
###################
echo
echo ">>> STEP 2"
echo

# Submit ACID transactions (NTRAN per STREAM)
#
for ((i=0;i<${STREAMS};i++)); do
	echo "  Starting $NTRAN ACID transactions for Stream ${i} at `date`"
	cat $OUTPUT_DIR/stream${i}_rnd | $IVWDIR/acid/dur/acidtrans_dur -b $DATABASE  > $OUTPUT_DIR/step2_stream${i}_output &
done

sleep 2

###################
# STEP 3
###################
echo
echo ">>> STEP 3"
echo

# wait until at least 100 transactions from each stream submitted
# in Step 2 have completed 
#
WAITMORE=1
while [ ${WAITMORE} -eq 1 ]; do
	WAITMORE=0
	
	for ((i=0;i<${STREAMS};i++)); do
		export COUNT${i}=`cat $OUTPUT_DIR/step2_stream${i}_output | grep -i "TPC-H ACID Transaction Ended" | wc -l`		
	done
	
	date
	for ((i=0;i<${STREAMS};i++)); do
		eval NTXN=\$COUNT$i 

		if [ $NTXN -lt 100 ]; then
			WAITMORE=1
		fi
		echo "Stream $i Completed $NTXN"
	done
	echo
	sleep 1
done

echo
echo "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
echo "!!!!! CAUSE THE FAILURE NOW `date` !!!!!"
echo "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
echo

sleep 1

echo "Waiting for streams to finish..."
echo "if they don't by themselves after the failure, kill -9 all"
echo "acidtrans_dur processes on the client before restarting the"
echo "Ingres server on the SUT."
echo
wait

echo
echo "Durability Test Ended at: `date`"
