#!/bin/bash

# This file is released under the terms of the Artistic License.
# Please see the file LICENSE, included in this package, for details.
#
# Copyright (c) 2010 Ingres Corp.

TOPDIR="`cat .topdir`"
IVWDIR="$TOPDIR/scripts/ingres_vectorwise"
source $TOPDIR/scripts/kit_profile || er "$0: could not source kit_profile"

unset SF
unset DSS_PATH
TESTDB=0
UPDSET=0
DEFAULT_CPUS=4
MERGE=1
SERIAL=0
START_UPDSET=1

if [ -r /proc/cpuinfo ]; then
	DEFAULT_CPUS=`cat /proc/cpuinfo | grep processor | wc -l`
	((DEFAULT_CPUS=$DEFAULT_CPUS*3/4))   # only use 3/4
fi
CPUS=$DEFAULT_CPUS

usage()
{
cat << EOF
$1

It generates test database data and/or Refresh Data sets. The default is to generate in parallel using $DEFAULT_CPUS CPUS (3/4 of the number of processors in the system); optionally, one can control the number of CPUS via -C flag.
For Resfreh Data sets one must pass the number of sets to generate (-r), optionally setting the starting set number (-ust).

usage: $0 flags

Required flags:
   -df    <dir>     location of the files to be generated
   -s     <number>  scale factor

At least one of following flags is required:
   -t               generate test database data files
   -r     <number>  generate passed number of Refresh Data Sets

Optional flags:
   -C     <N cpus>  (default=$DEFAULT_CPUS) number of processes to use for dbgen and sort on each table
   -T     <dir>     directory to use for sort space (default=$SORT_SPACE)
   -h               this help page
   -nm              do NOT merge <N cpus> files or each table back into one file
   -S               serialize intra-file generation, i.e., no pieces. Interfile is parallel.
			        this option overrides -C.
   -ust   <number>  start to generate refresh data sets from set #<number> (default=$START_UPDSET)
   
EOF
exit 1
}

while [ $# -ne 0 ]
do
    case $1 in
    -t)     TESTDB=1
	;;
    -df)    shift;
            DSS_PATH=$1
        ;;
    -r)     shift;
            UPDSET=$1
        ;;
    -s)     shift;
            SF=$1
        ;;
    -nm)    MERGE=0
        ;;
    -C)	    shift;
	    CPUS=$1
	;;
    -S)     SERIAL=1
	;;
    -T)	    shift;
	    SORT_SPACE=$1
	;;
    -ust)   shift;
        START_UPDSET=$1
    ;;
    *)      usage;
            ;;
    esac
    shift
done

if [ -z ${DSS_PATH} ]; then 
    usage "ERROR: location of the files is required."
fi

if [ -z ${SF} ]; then
    usage "ERROR: scale factor is required." 
fi

if [ ${TESTDB} -eq 0 -a $UPDSET -eq 0 ]; then
	usage "ERROR: choose one or more generation flags (-t, -r)"
fi	

if [ -z $SORT_SPACE ]; then
	SORT_SPACE=$DSS_PATH
fi

if [ ${TESTDB} -eq 1 ]; then
	echo "Generating data for scale factor $SF..."
	
	if [ ! -d $DSS_PATH ]; then
		mkdir -p $DSS_PATH || er "$DSS_PATH does not exist and could not be created"
	fi
			
	date
	pushd $DSS_PATH
	
	# do not parallelize the tiny tables, note this does not create foo.tbl.1 but foo.tbl
	${DBGEN} -s $SF -f -C 1 -T l || er "could not generate nation and region  data"

	# generate each table in prallel, in sequential order so only $CPUS cpus are in use
	for x in c L O P s S;do
	    if [ $SERIAL -eq 1 ]; then	
    		${DBGEN} -s $SF -f -C 1 -T $x || er "could not generate $x data" &
	    else
    		for ((piece=1; piece<=$CPUS; piece++));do
    			${DBGEN} -s $SF -f -C $CPUS -S $piece -T $x || er "could not generate $x data" &
    		done
    		wait
	    fi
	done
	wait
	chmod 664 ${DSS_PATH}/*.tbl*

	popd
	date

	# sort the scripts according to external parameter-file sort_param
	# it will automatically parallelize based on how many files there are for each table
	$IVWDIR/sort_flat_files.sh -d ${DSS_PATH} -T $SORT_SPACE || er "could not sort flat files"

	# merge flat files
	if [ $MERGE -gt 0 ]; then
		$IVWDIR/merge_flat_files.sh -d ${DSS_PATH} || er "could not sort flat files"
	fi
fi

if [ $UPDSET -gt 0 ]; then
	echo "Generating $UPDSET Refresh Data Set(s)"
	date
	pushd $DSS_PATH

    FINAL_UPDSET=`echo "$START_UPDSET+$UPDSET"|bc`
    if [ $SERIAL -eq 1 ]; then
        for ((piece=$START_UPDSET; piece<$FINAL_UPDSET; piece++)); do
            ${DBGEN} -s ${SF} -U $piece -S $piece \
                || er "could not generate Refresh Data Set for piece $piece" &
        done
    else
        for ((piece=$START_UPDSET; piece<$FINAL_UPDSET; piece++)); do
       	    ${DBGEN} -s ${SF} -U $piece -d $CPUS -i $CPUS -S $piece \
                || er "could not generate Refresh Data Set for piece $piece" &
        done
    fi
    wait
    
    chmod 664 *.tbl*
	popd
	date
fi

