#!/bin/bash

echo Script Arguments: $0 $*

#
# Copyright (c) 2010 Ingres Corp.
#

TOPDIR="`cat .topdir`"        
IVWDIR="$TOPDIR/scripts/ingres_vectorwise"
source $TOPDIR/scripts/kit_profile || er "$0: could not source kit_profile"

STATS=0
unset DATABASE
unset LOCATION
unset SF
OUTPUT_DIR="$IVWDIR/output"
USE_EXISTING_DB=0
PLOAD=0

usage()
{
cat << EOF
$1

usage: $0 flags

Required flags:
   -d     <name>    database name
   -df    <dir>     location of the database data files
   -s     <number>  scale factor

Optional flags:
   -loc   <name>    database location name
   -st              collect system statistics
   -nst             do not collect system statistics (default)
   -o     <dir>     where the tests artifacts will be stored (default=${OUTPUT_DIR})
   -nc              use existing database
   -h               this help page
   -par   <n>       do the load in parallel, default is serial (-par 0).
                       Flat files must have been previously split with the split.sh
                       script or dbgen -C -S using the same degree of parallelism.

EOF
exit 1
}

while [ $# -ne 0 ]
do
    case $1 in
    -d)     shift;
            DATABASE=$1
        ;;
    -df)    shift;
            DSS_PATH=$1
        ;;
    -loc)   shift;
            LOCATION=$1
        ;;
    -nc)    USE_EXISTING_DB=1
	;;
    -st)    STATS=1
        ;;
    -nst)   STATS=0
        ;;        
    -o)     shift;
            OUTPUT_DIR=$1
        ;;
    -par)   shift;
            PLOAD=$1
    ;;
    -s)     shift;
            SF=$1
        ;;
    *)      usage;
            ;;
    esac
    shift
done

if [ -z ${DATABASE} ]; then
   usage "ERROR: database name is required."   
fi

if [ -z ${DSS_PATH} ]; then
   usage "ERROR: location of the database data files is required."
fi

if [ -z ${SF} ]; then
   usage "ERROR: scale factor is required."   
fi

# Determine a unique identifer for this run.
if test -f $IVWDIR/run_number; then
        read RUN_NUMBER < $IVWDIR/run_number
else
        RUN_NUMBER=1
fi
TAG=$RUN_NUMBER

RUN_NUMBER=`expr $RUN_NUMBER + 1`
echo $RUN_NUMBER > $IVWDIR/run_number

#appending TAG into OUTPUT_DIR
OUTPUT_DIR="$OUTPUT_DIR/$TAG"
mkdir -p $OUTPUT_DIR
echo $OUTPUT_DIR > .output_dir

# initialize last_rfset_<database> file with 0
# This is used by RF scripts 
echo 0 > $IVWDIR/last_rfset_${DATABASE}

ingsetenv II_DATE_FORMAT YMD

if [ ${STATS} -eq 1 ]; then
    # Start collecting system statistics.
    $TOPDIR/scripts/start_sysstats.sh -o $OUTPUT_DIR || er "could not start sysstats"
fi
trap 'echo "*** Caught SIGTERM/SIGINT, stopping sysstats" && $TOPDIR/scripts/stop_sysstats.sh && exit 1' TERM INT QUIT ABRT

echo '======================================'
echo '          TPC-H Load Test'
echo '======================================'
echo


# Start of database load time
s_time=`$GTIME`
s_date=`date`
echo "Start time: $s_date"

# Start the database
$IVWDIR/start_db.sh

if [ ${USE_EXISTING_DB} -eq 0 ]; then
	# Drop database
	$IVWDIR/drop_db.sh ${DATABASE} 1

	# Create database
	if [ -z ${LOCATION} ]; then
	    $IVWDIR/create_db.sh -d ${DATABASE} || er "could not create db"
	else
	    $IVWDIR/create_db.sh -d ${DATABASE} -l ${LOCATION} || er "could not create db"
	fi
fi

# Create tables
cat $IVWDIR/import/header $IVWDIR/ddl/create_tables.sql | sql ${DATABASE}

# Create Indexes
cat $IVWDIR/import/header $IVWDIR/ddl/create_indexes.sql | sql ${DATABASE} 

# Load data

header="$IVWDIR/import/header"

# do nation and region in the normal way
for t in region nation; do
    f="$IVWDIR/import/${t}.sql"
    echo "Loading $f..."
    pushd $DSS_PATH
    echo `cat $header $f` '\p\g' | sql $DATABASE
    popd
done

# load the others in parallel if reqeusted
if [ $PLOAD -eq 0 ]; then
    for t in supplier part partsupp customer orders lineitem; do
        f="$IVWDIR/import/${t}.sql"
        echo "Loading $f..."
        pushd $DSS_PATH
        echo `cat $header $f` '\p\g' | sql $DATABASE
        popd
    done
else
    tmp=`mktemp`
    for t in supplier part partsupp customer orders lineitem; do

        # create temp tables and load them all in parallel
        for ((p=1; p<=PLOAD; p++));do
            echo "Loading $t${p}..."
 
           # note the $p here is replaced by the shell with the actual number
            echo "s/NN/${p}/" > $tmp$p
            sed <$IVWDIR/ddl_p/${t}.sql -f $tmp$p | sql $DATABASE
            pushd $DSS_PATH
            sed <$IVWDIR/import_p/${t}.sql -f $tmp$p | sql $DATABASE &
            popd
        done
    done
    time wait
    rm -f ${tmp}*
    echo Temp table loads done at `date`

    # loads into parallel temp tables were all in parallel but, the combines are serial
    for t in supplier part partsupp customer orders lineitem; do
        echo -n "insert into $t select * from ${t}1 " >$tmp
        for ((p=2; p<=PLOAD; p++));do
            echo -n " union ALL select * from $t$p " >>$tmp
        done
        echo ";\p\g" >>$tmp

        # drop the tables as soon as we are done with them
        for ((p=1; p<=PLOAD; p++));do
            echo "drop table $t${p};\p\g" >>$tmp
        done
        sql <$tmp $DATABASE
    done
    echo Insert Select tables done at `date`
fi

# Generate statistics
$IVWDIR/update_statistics.sh ${DATABASE}

# Grant rights to PUBLIC
echo `cat $header $IVWDIR/ddl/grant.sql` '\p\g' | sql ${DATABASE}

if [ ${STATS} -eq 1 ]; then
    # Stop collecting system statistics.
    echo "Stop collecting system statistics."
    $TOPDIR/scripts/stop_sysstats.sh || er "$0: could not run stop_sysstats.sh"
fi

# End of database load time
e_time=`$GTIME`
e_date=`date`

# time of the end of the database load is used to seed the random number generator
# for the substitution parameter
SEED=`$TOPDIR/scripts/init_seed.sh`
echo ${SEED} > $IVWDIR/seed_file

echo '=========================='
echo '=    Load Completed      ='
echo '=========================='
echo ' '

# calculate and report load time
#
diff_time=`calc $e_time-$s_time`
echo "Database Name @SF: ${DATABASE} @ SF${SF}"
echo "Start time of database load: $s_date" >> $OUTPUT_DIR/load_report.txt
echo "End time of database load: $e_date" >> $OUTPUT_DIR/load_report.txt
echo "Database Load Time: $diff_time seconds" >> $OUTPUT_DIR/load_report.txt

# Store Seed and DBGEN/QGEN versions
echo "Seed: ${SEED}" >> $OUTPUT_DIR/load_report.txt
echo "DBGEN version: `$DBGEN -h 2>&1 | sed 1q`" >> $OUTPUT_DIR/load_report.txt
echo "QGEN version: `$QGEN -h 2>&1 | sed 1q`" >> $OUTPUT_DIR/load_report.txt

# Get and report cardinality of tables (as required by clause 8.3.5.1)
#
host=`uname -n`
echo " " | tee -a $OUTPUT_DIR/load_report.txt
echo "Tables Cardinality" | tee -a $OUTPUT_DIR/load_report.txt
echo "==================" | tee -a $OUTPUT_DIR/load_report.txt
grep -v '#' $IVWDIR/location_names.$host | (while [ 1 ]; do
    read ignore_loc ignore_area ignore_disk ignore_pct table
    if [ $? -ne 0 ]; then
        break
    fi

    echo "select count(*) from ${table}\g"| sql -s ${DATABASE} > $OUTPUT_DIR/${table}.count

        # compare cardinality to reference data
        if [ -f $IVWDIR/reference/cardinality.sf${SF}/${table} ]; then
                DIFF=`cat ${OUTPUT_DIR}/${table}.count | sed -e "1,2 d" | grep -v +-*+ | diff -b - $IVWDIR/reference/cardinality.sf${SF}/${table}`
                if [ -n "$DIFF" ]; then
                        echo "${table}" | awk '{printf "%-10s ...FAILED\n",$1}' | tee -a  $OUTPUT_DIR/load_report.txt
                else
                        echo "${table}" | awk '{printf "%-10s ...OK\n",$1}' | tee -a $OUTPUT_DIR/load_report.txt
                fi
        else
                echo "${table}" | awk '{printf "%-10s ...No reference data found\n",$1}' | tee -a $OUTPUT_DIR/load_report.txt
         fi
done )
echo "==================" | tee -a $OUTPUT_DIR/load_report.txt
echo " " | tee -a $OUTPUT_DIR/load_report.txt
cat $OUTPUT_DIR/*.count >> $OUTPUT_DIR/load_report.txt

head -n 5 $OUTPUT_DIR/load_report.txt

echo " "
echo "See full report in $OUTPUT_DIR/load_report.txt"
echo "output dir #${TAG}"
